#include<sstream>
#include<iostream>

#include "TaskListNode.hh"
#include "PlanTreeTaskListNode.hh"
#include "tools.hh"

using namespace std;

unsigned long int TaskListNode::count = START_COUNT_ID; // init of count

/**
 * constructor: makes a copy of PTTLn under a TaskListNode form, TaskListNode IDs will be updated
 * according count, status is set to #TODO and cost is set to -1
 */
TaskListNode::TaskListNode(PlanTreeTaskListNode const& PTTLn)
	: instance(PTTLn.getInstance()), ID(count++), status(TODO), cost(-1.) {}

/**
 * add s in successors if it is not already in it, else update the link type according link_type value
 */
void TaskListNode::naiveAddSuccessor(TaskListNode* s, string const& link_type) {
	vector<TaskListNode*>::iterator it;
	if(s == NULL)
		cerr << "ERROR: trying to add a not existing TaskListNode in a TaskListNode successors" << endl;
	else if( find(predecessors.begin(), predecessors.end(), s) != predecessors.end() ) {
		// s is already in predecessors (it will make a loop)
		cerr << "ERROR: trying to add TaskListNode " << s->getID() << " in TaskListNode " << ID;
		cerr << " successors whereas it is already in predecessors" << endl;
	}
	else {
		it = find(successors.begin(), successors.end(), s);
		if( it != successors.end() ) // s is already in successors, we update link type
			successors_links[std::distance(successors.begin(), it)] = link_type;
		else { // s is not already in successors
			successors.push_back(s);
			successors_links.push_back(link_type);
		}
	}
}

/**
 * add p in predecessors if it is not already in it, else update the link type according link_type value
 */
void TaskListNode::naiveAddPredecessor(TaskListNode* p, string const& link_type) {
	vector<TaskListNode*>::iterator it;
	if(p == NULL)
		cerr << "ERROR: trying to add a not existing TaskListNode in a TaskListNode predecessors" << endl;
	else if( find(successors.begin(), successors.end(), p) != successors.end() ) {
		// s is already in successors (it will make a loop)
		cerr << "ERROR: trying to add TaskListNode " << p->getID() << " in TaskListNode " << ID;
		cerr << " predecessors whereas it is already in successors" << endl;
	}
	else {
		it = find(predecessors.begin(), predecessors.end(), p);
		if( it != predecessors.end() ) // s is already in predecessors, we update link type
			predecessors_links[std::distance(predecessors.begin(), it)] = link_type;
		else { // s is not already in predecessors
			predecessors.push_back(p);
			predecessors_links.push_back(link_type);
		}
	}
}

/**
 * scans all predecessors and if one is not realized returns false
 */
bool TaskListNode::predecessorsAllDone() const {
	if(noPredecessors()) return false;
	for(unsigned int i=0 ; i<predecessors.size() ; ++i)
		if( !(predecessors[i]->getStatus() == DONE) ) return false;
	return true;
}

/**
 * return true if TLn has the same successors as the current TaskListNode (the comparison of successors
 * is based on the tasks transformed into strings), it does not care about link types, status or IDs
 */
bool TaskListNode::hasSameSuccessors(TaskListNode const* TLn) const {
	using namespace HATP_TOOLS;
	unsigned int i;
	vector<TaskListNode*> TLnSucc = TLn->getSuccessors();
	vector<string> vec1(successors.size()), vec2(TLnSucc.size());
	for(i=0 ; i<successors.size() ; ++i) vec1[i] = successors[i]->getInstance().toString();
	for(i=0 ; i<TLnSucc.size() ; ++i) vec2[i] = TLnSucc[i]->getInstance().toString();
	return (vec1 == vec2);
}

/**
 * return true if TLn has the same predecessors as the current TaskListNode (the comparison of predecessors
 * is based on the tasks transformed into strings), it does not care about link types, status or IDs
 */
bool TaskListNode::hasSamePredecessors(TaskListNode const* TLn) const {
	using namespace HATP_TOOLS;
	unsigned int i;
	vector<TaskListNode*> TLnPred = TLn->getPredecessors();
	vector<string> vec1(predecessors.size()), vec2(TLnPred.size());
	for(i=0 ; i<predecessors.size() ; ++i) vec1[i] = predecessors[i]->getInstance().toString();
	for(i=0 ; i<TLnPred.size() ; ++i) vec2[i] = TLnPred[i]->getInstance().toString();
	return (vec1 == vec2);
}

/**
 * convert TaskListNode to a string and returns it. Pproduced string : _Task_ {status: _status_}
 */
string TaskListNode::toString() const {
	ostringstream st;
	st << instance.toString() << " {status: " << (status==0?"TODO":(status==1?"DONE":"TODO_LOCKED")) << ", hasPred:";
	st << std::boolalpha << (!noPredecessors());
	if(!noPredecessors()) st << ", allPredDone:" << predecessorsAllDone();
	st << "}" ;
	string s(st.str());
	return s;
}

/**
 * destroy all links in predecessors and successors which are of type sLinkType
 */
void TaskListNode::destroyLinksOfType(const string& sLinkType) {
	unsigned int i=0;
	for(i=0 ; i<predecessors_links.size() ; ++i)
		if(predecessors_links[i] == sLinkType) this->RemovePredecessor(predecessors[i]);
	for(i=0 ; i<successors_links.size() ; ++i)
		if(successors_links[i] == sLinkType) this->RemoveSuccessor(successors[i]);
}

