#if !defined HATP_WORLDBASEAGENT_HH_
#define HATP_WORLDBASEAGENT_HH_

#include<string>
#include<vector>
#include<map>

#include "Entity.hh"

/** Represent what an agent knows about the world (and the entities).
 * This is related to the belief management integrated in HATP.
 */
class WorldBaseAgent {

private:
	/** Is used when returning a reference to an entity after an error in operator[](const char* c). */
	Entity defaultEntity;
	/** Contains the set of entities that the agent knows. */
	std::map<std::string, Entity*> database;

public:
	explicit WorldBaseAgent();
	WorldBaseAgent(WorldBaseAgent const& wb);
	const std::map<std::string, Entity*>& getDatabase() const;
	void add(Entity const& e);
	bool containEntity(std::string const& sName) const;
	bool containEntity(std::string const& sName, std::string const& sType) const;
	Entity* find(std::string const& sName);
	Entity const* find(std::string const& sName) const;
	Entity* find(std::string const& sName, std::string const& sType);
	Entity const* find(std::string const& sName, std::string const& sType) const;
	std::vector<Entity*> findByType(std::string const& sType) const;
	std::vector<Entity*> findByAttribute(std::string const& attrName) const;
	std::string toString() const;
	Entity& operator[](const char* c);
	Entity& operator[](std::string const& s);
	Entity const& operator[](const char* c) const;
	Entity const& operator[](std::string const& s) const;
	WorldBaseAgent& operator=(WorldBaseAgent const& wb);
	void clearAllDynamicVectors();
	~WorldBaseAgent();
};

/** Default constructor.
 * All constructors set defaultEntity to an empty Entity.
 */
inline WorldBaseAgent::WorldBaseAgent() : defaultEntity(Entity()) {}

/** Copy constructor.
 * All constructors set defaultEntity to an empty Entity.
 * @param wb Agent database to copy
 */
inline WorldBaseAgent::WorldBaseAgent(WorldBaseAgent const& wb) : defaultEntity(Entity()) {
	std::map<Entity*, Entity*> oldToNew; // to keep correspondence
	Entity *e;
	std::map<std::string, Entity*> const& tmp=wb.getDatabase();
	for(std::map<std::string, Entity*>::const_iterator it = tmp.begin() ; it!=tmp.end() ; ++it) {
		e = new Entity(*(it->second));
		database[it->first] = e;
		oldToNew[it->second] = e;
	}
	// we need to update the Entity* attributes
	for(std::map<std::string, Entity*>::iterator itD = database.begin() ; itD!=database.end() ; ++itD)
		itD->second->updateAttrs(oldToNew);
}

/** Gives a reference to the entity database for this agent.
 * @return Entity database
 */
inline const std::map<std::string, Entity*>& WorldBaseAgent::getDatabase() const {
	return database;
}

/** Assignement operator.
 * @param wb WorldBaseAgent to copy
 * @return Reference to this
 */
inline WorldBaseAgent& WorldBaseAgent::operator=(WorldBaseAgent const& wb) {
	// we empty database
	for(std::map<std::string, Entity*>::iterator itD = database.begin() ; itD != database.end() ; ++itD)
		delete itD->second;
	database.clear();
	std::map<Entity*, Entity*> oldToNew; // to keep correspondence
	Entity *e;
	std::map<std::string, Entity*> tmp=wb.getDatabase();
	for(std::map<std::string, Entity*>::const_iterator it = tmp.begin() ; it!=tmp.end() ; ++it) {
		e = new Entity(*(it->second));
		database[it->first] = e;
		oldToNew[it->second] = e;
	}
	// we need to update the Entity* attributes
	for(std::map<std::string, Entity*>::iterator itD = database.begin() ; itD!=database.end() ; ++itD)
		itD->second->updateAttrs(oldToNew);
	return *this;
}

#endif
